<?php

namespace App\Http\Controllers\Client;

use App\Http\Controllers\Controller;
use App\Http\Requests\LoginRequest;
use App\Http\Requests\RecoverAccountRequest;
use App\Http\Requests\RegisterRequest;
use App\Http\Requests\ResetPasswordRequest;
use Illuminate\Auth\Events\PasswordReset;
use Illuminate\Auth\Events\Registered;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use App\Models\User;
use Illuminate\Support\Facades\Password;

class AuthController extends Controller
{
    /**
     *  @OA\Post(
     *    tags={"Clients:Authentication"},
     *    path="/auth/login",
     *    summary="Login to the API",
     *    @OA\RequestBody(
     *      ref="#/components/requestBodies/Login"
     *    ),
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="422",
     *      ref="#/components/responses/422"
     *    ),
     *    @OA\Response(
     *      response="200",
     *      description="OK",
     *      @OA\JsonContent(
     *        @OA\Property(
     *          property="token",
     *          type="string",
     *          description="Token to use as Bearer token",
     *        ),
     *        @OA\Property(
     *          property="user",
     *          type="object",
     *          allOf={
     *            @OA\Schema(ref="#/components/schemas/User"),
     *            @OA\Schema(
     *              @OA\Property(type="string", property="email", format="email"),
     *            )
     *          }
     *        ),
     *      ),
     *    ),
     *  )
     */
    public function login(LoginRequest $request)
    {
        $token = Auth::guard('client')->attempt($request->validated());

        if (!$token) {
            return response()->json([
                'message' => 'Unauthorized',
            ], 401);
        }

        $user = Auth::guard('client')->user();

        // TODO: RE-enable when it works again
        /*
        if (!$user->hasVerifiedEmail()) {
            return response()->json([
                'message' => 'Please verify your email',
            ], 401);
        }
        */

        return response()->json([
            'user' => $user->makeVisible('email'),
            'token' => $token,
        ]);
    }

    /**
     *  @OA\Post(
     *    tags={"Clients:Authentication"},
     *    path="/auth/register",
     *    summary="Register to the API",
     *    @OA\RequestBody(
     *      ref="#/components/requestBodies/Register"
     *    ),
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="422",
     *      ref="#/components/responses/422"
     *    ),
     *    @OA\Response(
     *      response="204",
     *      description="OK"
     *    ),
     *  )
     */
    public function register(RegisterRequest $request)
    {
        $user = User::create([
            'username' => $request->username,
            'email' => $request->email,
            'password' => Hash::make($request->password),
        ]);

        event(new Registered($user));

        return response()->noContent();
    }

    /**
     *  @OA\Post(
     *    tags={"Clients:Authentication"},
     *    path="/auth/logout",
     *    summary="Logout of the API",
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="204",
     *      description="OK",
     *    ),
     *    security={{"jwt":{}}},
     *  )
     */
    public function logout()
    {
        Auth::guard('client')->logout();
        return response()->noContent();
    }

    /**
     *  @OA\Post(
     *    tags={"Clients:Authentication"},
     *    path="/auth/refresh",
     *    summary="Refresh an API token",
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="200",
     *      description="OK",
     *      @OA\JsonContent(
     *        @OA\Property(
     *          property="token",
     *          type="string",
     *          description="Token to use as Bearer token",
     *        ),
     *        @OA\Property(
     *          property="user",
     *          type="object",
     *          allOf={
     *            @OA\Schema(ref="#/components/schemas/User"),
     *            @OA\Schema(
     *              @OA\Property(type="string", property="email", format="email"),
     *            )
     *          }
     *        ),
     *      ),
     *    ),
     *    security={{"jwt":{}}},
     *  )
     */
    public function refresh()
    {
        return response()->json([
            'user' => Auth::guard('client')->user()->makeVisible('email'),
            'token' => Auth::guard('client')->refresh(),
        ]);
    }

    /**
     *  @OA\Post(
     *    tags={"Clients:Authentication"},
     *    path="/auth/recover",
     *    summary="Recover an account",
     *    @OA\RequestBody(
     *      ref="#/components/requestBodies/RecoverAccount"
     *    ),
     *    @OA\Response(
     *      response="422",
     *      ref="#/components/responses/422"
     *    ),
     *    @OA\Response(
     *      response="429",
     *      description="Too Many Requests",
     *      @OA\JsonContent(
     *        @OA\Property(
     *          property="message",
     *          type="string",
     *        ),
     *      )
     *    ),
     *    @OA\Response(
     *      response="204",
     *      description="OK",
     *    ),
     *  )
     */
    public function recover(RecoverAccountRequest $request)
    {
        $status = Password::sendResetLink(
            $request->only('email')
        );

        if ($status === Password::RESET_LINK_SENT) {
            return response()->noContent();
        } else if ($status === Password::INVALID_USER) {
            return response()->json([
                'message' => __($status),
            ], 401);
        } else if ($status === Password::RESET_THROTTLED) {
            return response()->json([
                'message' => __($status),
            ], 429);
        }
    }

    /**
     *  @OA\Post(
     *    tags={"Clients:Authentication"},
     *    path="/auth/reset-password",
     *    summary="Recover an account",
     *    @OA\RequestBody(
     *      ref="#/components/requestBodies/ResetPassword"
     *    ),
     *    @OA\Response(
     *      response="401",
     *      ref="#/components/responses/401"
     *    ),
     *    @OA\Response(
     *      response="422",
     *      ref="#/components/responses/422"
     *    ),
     *    @OA\Response(
     *      response="429",
     *      description="Too Many Requests",
     *      @OA\JsonContent(
     *        @OA\Property(
     *          property="message",
     *          type="string",
     *        ),
     *      )
     *    ),
     *    @OA\Response(
     *      response="204",
     *      description="OK",
     *    ),
     *  )
     */
    public function reset_password(ResetPasswordRequest $request)
    {
        $status = Password::reset(
            $request->only('email', 'password', 'token'),
            function (User $user, string $password) {
                $user->forceFill([
                    'password' => Hash::make($password)
                ]);

                $user->save();

                event(new PasswordReset($user));
            }
        );

        if ($status == Password::INVALID_USER || $status == Password::INVALID_TOKEN) {
            return response()->json([
                "message" => __($status)
            ], 401);
        } else if ($status == Password::PASSWORD_RESET) {
            return response()->noContent();
        }
    }
}
