<?php

namespace Tests\Feature;

use App\Models\Game;
use App\Models\Server;
use App\Models\User;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithFaker;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Tests\TestCase;

class GameCreationTest extends TestCase
{
    use RefreshDatabase;
    
    public function test_good()
    {
        $user1 = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $user2 = User::create([
            'username' => 'test2',
            'email' => 'test2@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user1->id,
        ]);
        $server->official = true;
        $server->save();

        $user1->server_id = $server->id;
        $user1->server_state = 'connected';
        $user1->elo = 1200;
        $user1->save();

        $user2->server_id = $server->id;
        $user2->server_state = 'connected';
        $user1->elo = 1000;
        $user2->save();

        $token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->postJson('/api/game', [
            "mode" => "ffa",
            "users" => [[
                "id" => $user1->id
            ], [
                "id" => $user2->id
            ]]
        ]);
        $response->assertStatus(201);
        $response->assertJsonStructure([
            "game" => [
                "id",
                "server_id",
                "mode",
                "created_at",
                "updated_at"
            ]
        ]);
        $this->assertDatabaseCount("games", 1);
        $this->assertDatabaseCount("users_games", 2);
        $id = $response->json("game.id");

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->putJson('/api/game/' . $id, [
            "state" => "finished",
            "users" => [[
                "id" => $user1->id,
                "rank" => 1
            ], [
                "id" => $user2->id,
                "rank" => 0
            ]]
        ]);
        $response->assertStatus(200);

        $this->assertEquals(1, Game::first()->users()->findOrFail($user1->id)->pivot->rank);
        $this->assertEquals(1, Game::first()->users()->findOrFail($user1->id)->rank);
        $this->assertGreaterThan(1200, Game::first()->users()->findOrFail($user1->id)->elo);
        $this->assertEquals(2, Game::first()->users()->findOrFail($user2->id)->pivot->rank);
        $this->assertEquals(2, Game::first()->users()->findOrFail($user2->id)->rank);
        $this->assertLessThan(1000, Game::first()->users()->findOrFail($user2->id)->elo);
    }

    public function test_good_four_players()
    {
        $user1 = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $user2 = User::create([
            'username' => 'test2',
            'email' => 'test2@localhost',
            'password' => Hash::make('test'),
        ]);

        $user3 = User::create([
            'username' => 'test3',
            'email' => 'test3@localhost',
            'password' => Hash::make('test'),
        ]);

        $user4 = User::create([
            'username' => 'test4',
            'email' => 'test4@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user1->id,
        ]);
        $server->official = true;
        $server->save();

        $user1->server_id = $server->id;
        $user1->server_state = 'connected';
        $user1->elo = 1200;
        $user1->save();

        $user2->server_id = $server->id;
        $user2->server_state = 'connected';
        $user2->elo = 1400;
        $user2->save();

        $user3->server_id = $server->id;
        $user3->server_state = 'connected';
        $user3->elo = 800;
        $user3->save();

        $user4->server_id = $server->id;
        $user4->server_state = 'connected';
        $user4->elo = 900;
        $user4->save();

        $token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->postJson('/api/game', [
            "mode" => "ffa",
            "users" => [[
                "id" => $user1->id
            ], [
                "id" => $user2->id
            ], [
                "id" => $user3->id
            ], [
                "id" => $user4->id
            ]]
        ]);
        $response->assertStatus(201);
        $response->assertJsonStructure([
            "game" => [
                "id",
                "server_id",
                "mode",
                "created_at",
                "updated_at"
            ]
        ]);
        $this->assertDatabaseCount("games", 1);
        $this->assertDatabaseCount("users_games", 4);
        $id = $response->json("game.id");

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->putJson('/api/game/' . $id, [
            "state" => "finished",
            "users" => [[
                "id" => $user1->id,
                "rank" => 1
            ], [
                "id" => $user2->id,
                "rank" => 0
            ], [
                "id" => $user3->id,
                "rank" => 3
            ], [
                "id" => $user4->id,
                "rank" => 2
            ]]
        ]);
        $response->assertStatus(200);

        $this->assertEquals(Game::first()->users()->findOrFail($user1->id)->pivot->rank, 3);
        $this->assertEquals(Game::first()->users()->findOrFail($user2->id)->pivot->rank, 4);
        $this->assertEquals(Game::first()->users()->findOrFail($user3->id)->pivot->rank, 1);
        $this->assertEquals(Game::first()->users()->findOrFail($user4->id)->pivot->rank, 2);
    }

    public function test_good_four_players_teams()
    {
        $user1 = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $user2 = User::create([
            'username' => 'test2',
            'email' => 'test2@localhost',
            'password' => Hash::make('test'),
        ]);

        $user3 = User::create([
            'username' => 'test3',
            'email' => 'test3@localhost',
            'password' => Hash::make('test'),
        ]);

        $user4 = User::create([
            'username' => 'test4',
            'email' => 'test4@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user1->id,
        ]);
        $server->official = true;
        $server->save();

        $user1->server_id = $server->id;
        $user1->server_state = 'connected';
        $user1->save();

        $user2->server_id = $server->id;
        $user2->server_state = 'connected';
        $user2->save();

        $user3->server_id = $server->id;
        $user3->server_state = 'connected';
        $user3->save();

        $user4->server_id = $server->id;
        $user4->server_state = 'connected';
        $user4->save();

        $token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->postJson('/api/game', [
            "mode" => "team",
            "users" => [[
                "id" => $user1->id,
                "team_id" => 0
            ], [
                "id" => $user2->id,
                "team_id" => 0
            ], [
                "id" => $user3->id,
                "team_id" => 1
            ], [
                "id" => $user4->id,
                "team_id" => 1
            ]]
        ]);
        $response->assertStatus(201);
        $response->assertJsonStructure([
            "game" => [
                "id",
                "server_id",
                "mode",
                "created_at",
                "updated_at"
            ]
        ]);
        $this->assertDatabaseCount("games", 1);
        $this->assertDatabaseCount("users_games", 4);
        $id = $response->json("game.id");

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->putJson('/api/game/' . $id, [
            "state" => "finished",
            "users" => [[
                "id" => $user1->id,
                "rank" => 0
            ], [
                "id" => $user2->id,
                "rank" => 3
            ], [
                "id" => $user3->id,
                "rank" => 1
            ], [
                "id" => $user4->id,
                "rank" => 2
            ]]
        ]);
        $response->assertStatus(200);

        $this->assertEquals(Game::first()->users()->findOrFail($user1->id)->pivot->rank, 2);
        $this->assertEquals(Game::first()->users()->findOrFail($user2->id)->pivot->rank, 1);
        $this->assertEquals(Game::first()->users()->findOrFail($user3->id)->pivot->rank, 4);
        $this->assertEquals(Game::first()->users()->findOrFail($user4->id)->pivot->rank, 3);
    }

    public function test_user_not_connected()
    {
        $user1 = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $user2 = User::create([
            'username' => 'test2',
            'email' => 'test2@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user1->id,
        ]);

        $user1->server_id = $server->id;
        $user1->server_state = 'connected';
        $user1->save();

        $token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->postJson('/api/game', [
            "mode" => "ffa",
            "users" => [[
                "id" => $user1->id
            ], [
                "id" => $user2->id
            ]]
        ]);

        $response->assertStatus(403);
        $this->assertDatabaseCount("games", 0);
        $this->assertDatabaseCount("users_games", 0);
    }

    public function test_not_enough_users()
    {
        $user1 = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user1->id,
        ]);

        $user1->server_id = $server->id;
        $user1->server_state = 'connected';
        $user1->save();

        $token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->postJson('/api/game', [
            "users" => [[
                "id" => $user1->id
            ]]
        ]);

        $response->assertStatus(422);
        $this->assertDatabaseCount("games", 0);
        $this->assertDatabaseCount("users_games", 0);
    }

    public function test_same_users()
    {
        $user1 = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user1->id,
        ]);

        $user1->server_id = $server->id;
        $user1->server_state = 'connected';
        $user1->save();

        $token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->postJson('/api/game', [
            "users" => [[
                "id" => $user1->id
            ], [
                "id" => $user1->id
            ]]
        ]);

        $response->assertStatus(422);
        $this->assertDatabaseCount("games", 0);
        $this->assertDatabaseCount("users_games", 0);
    }

    public function test_wrong_users()
    {
        $user1 = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $user2 = User::create([
            'username' => 'test2',
            'email' => 'test2@localhost',
            'password' => Hash::make('test'),
        ]);

        $user3 = User::create([
            'username' => 'test3',
            'email' => 'test3@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user1->id,
        ]);

        $user1->server_id = $server->id;
        $user1->server_state = 'connected';
        $user1->save();

        $user2->server_id = $server->id;
        $user2->server_state = 'connected';
        $user2->save();

        $user3->server_id = $server->id;
        $user3->server_state = 'connected';
        $user3->save();

        $token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->postJson('/api/game', [
            "mode" => "ffa",
            "users" => [[
                "id" => $user1->id
            ], [
                "id" => $user2->id
            ]]
        ]);
        $response->assertStatus(201);
        $response->assertJsonStructure([
            "game" => [
                "id",
                "server_id",
                "mode",
                "created_at",
                "updated_at"
            ]
        ]);
        $this->assertDatabaseCount("games", 1);
        $this->assertDatabaseCount("users_games", 2);
        $id = $response->json("game.id");

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->putJson('/api/game/' . $id, [
            "state" => "finished",
            "users" => [[
                "id" => $user1->id,
                "rank" => 1
            ], [
                "id" => $user3->id,
                "rank" => 0
            ]]
        ]);
        $response->assertStatus(403);
    }

    public function test_user_connected_to_other_server()
    {
        $user1 = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $user2 = User::create([
            'username' => 'test2',
            'email' => 'test2@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user1->id,
        ]);

        $server2 = Server::create([
            'login' => 'test2',
            'password' => Hash::make('test'),
            'owner_id' => $user1->id,
        ]);

        $user1->server_id = $server->id;
        $user1->server_state = 'connected';
        $user1->save();

        $user2->server_id = $server2->id;
        $user2->server_state = 'connected';
        $user2->save();

        $token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->postJson('/api/game', [
            "mode" => "ffa",
            "users" => [[
                "id" => $user1->id
            ], [
                "id" => $user2->id
            ]]
        ]);

        $response->assertStatus(403);
        $this->assertDatabaseCount("games", 0);
        $this->assertDatabaseCount("users_games", 0);
    }

    public function test_update_game_of_other_server()
    {
        $user1 = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $user2 = User::create([
            'username' => 'test2',
            'email' => 'test2@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user1->id,
        ]);

        $server2 = Server::create([
            'login' => 'test2',
            'password' => Hash::make('test'),
            'owner_id' => $user1->id,
        ]);

        $user1->server_id = $server->id;
        $user1->server_state = 'connected';
        $user1->save();

        $user2->server_id = $server->id;
        $user2->server_state = 'connected';
        $user2->save();

        $token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->postJson('/api/game', [
            "mode" => "ffa",
            "users" => [[
                "id" => $user1->id
            ], [
                "id" => $user2->id
            ]]
        ]);
        $response->assertStatus(201);
        $response->assertJsonStructure([
            "game" => [
                "id",
                "server_id",
                "created_at",
                "updated_at"
            ]
        ]);
        $this->assertDatabaseCount("games", 1);
        $this->assertDatabaseCount("users_games", 2);

        $id = $response->json("game.id");

        $token2 = Auth::guard('server')->login($server2);
        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token2
        ])->putJson('/api/game/' . $id, [
            "state" => "finished",
            "users" => [[
                "id" => $user1->id,
                "rank" => 0
            ], [
                "id" => $user2->id,
                "rank" => 1
            ]]
        ]);
        $response->assertStatus(404);
    }
}
