<?php

namespace Tests\Feature;

use App\Models\Server;
use App\Models\User;
use App\Models\Game;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithFaker;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Tests\TestCase;

class ServerTest extends TestCase
{
    use RefreshDatabase;

    public function test_index()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        Server::create([
            "login" => "test",
            "password" => "test",
            "owner_id" => $user->id
        ]);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/server');
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "servers" => [
                '*' => [
                    "id",
                    "login",
                    "official",
                    "created_at",
                    "updated_at",
                    "owner_id"
                ]
            ]
        ]);
    }

    public function test_show_not_found()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/server/1');
        $response->assertStatus(404);
    }

    public function test_show_good()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);


        $server = Server::create([
            "login" => "test",
            "password" => "test",
            "owner_id" => $user->id
        ]);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/server/' . $server->id);
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "server" => [
                "id",
                "login",
                "official",
                "created_at",
                "updated_at",
                "owner_id"
            ]
        ]);
    }
    public function test_index_games_server_not_found()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/server/1/game');
        $response->assertStatus(404);
    }

    public function test_index_games_good()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $server = Server::create([
            "login" => "test",
            "password" => "test",
            "owner_id" => $user->id
        ]);

        $game = Game::create([
            "server_id" => $server->id,
            "mode" => "ffa"
        ]);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/server/' . $server->id . '/game');
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "games" => [
                '*' => [
                    "id",
                    "server_id",
                    "mode",
                    "created_at",
                    "updated_at"
                ]
            ]
        ]);
    }

    public function test_show_games_server_not_found()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/server/1/game/1');
        $response->assertStatus(404);
    }

    public function test_show_games_good()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $server = Server::create([
            "login" => "test",
            "password" => "test",
            "owner_id" => $user->id
        ]);

        $game = Game::create([
            "server_id" => $server->id,
            "mode" => "ffa"
        ]);
        $game->state = 'finished';
        $game->save();

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/server/' . $server->id . '/game/' . $game->id);
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "game" => [
                "id",
                "server_id",
                "mode",
                "created_at",
                "updated_at"
            ]
        ]);
    }


    public function test_delete_not_found()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->delete('/api/server/1');
        $response->assertStatus(404);
    }

    public function test_delete_forbidden()
    {
        $user1 = User::create([
            'username' => 'test1',
            'email' => 'test1@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            "login" => "test",
            "password" => "test",
            "owner_id" => $user1->id
        ]);

        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->delete('/api/server/' . $server->id);
        $response->assertStatus(403);
    }

    public function test_delete_good()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);


        $server = Server::create([
            "login" => "test",
            "password" => "test",
            "owner_id" => $user->id
        ]);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->delete('/api/server/' . $server->id);
        $response->assertStatus(204);
        $this->assertDatabaseCount('servers', 0);
    }

    public function test_store_empty()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->post('/api/server');
        $response->assertStatus(422);
        $this->assertDatabaseCount('servers', 0);
    }

    public function test_store_login_already_used()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $server = Server::create([
            "login" => "test",
            "password" => "test",
            "owner_id" => $user->id
        ]);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->post('/api/server', [
            "login" => "test",
            "password" => "testtest",
        ]);
        $response->assertStatus(422);
        $this->assertDatabaseCount('servers', 1);
    }

    public function test_store_good()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->post('/api/server', [
            "login" => "test",
            "password" => "testtest",
        ]);
        $response->assertStatus(201);
        $this->assertDatabaseCount('servers', 1);
    }

    public function test_update_empty()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $server = Server::create([
            "login" => "test",
            "password" => "test",
            "owner_id" => $user->id
        ]);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->put('/api/server/' . $server->id);
        $response->assertStatus(204);
    }

    public function test_update_login_already_used()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $server = Server::create([
            "login" => "test1",
            "password" => "test",
            "owner_id" => $user->id
        ]);

        $server = Server::create([
            "login" => "test",
            "password" => "test",
            "owner_id" => $user->id
        ]);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->put('/api/server/' . $server->id, [
            "login" => "test1"
        ]);
        $response->assertStatus(422);
    }

    public function test_update_forbidden()
    {
        $user1 = User::create([
            'username' => 'test1',
            'email' => 'test1@localhost',
            'password' => Hash::make('test'),
        ]);
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $server = Server::create([
            "login" => "test",
            "password" => "test",
            "owner_id" => $user1->id
        ]);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->put('/api/server/' . $server->id, [
            "login" => "test1"
        ]);
        $response->assertStatus(403);
    }

    public function test_update_good()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $server = Server::create([
            "login" => "test",
            "password" => "test",
            "owner_id" => $user->id
        ]);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->put('/api/server/' . $server->id, [
            "login" => "testtest",
            "password" => "testtest"
        ]);
        $response->assertStatus(204);
    }
}
